"""This constructs the type of a buildable app in the
current context
"""


from koil.composition import Composition
from typing import Any

NotAvailable = Any


try:
    from arkitekt.apps.rekuest import ArkitektRekuest
except ImportError as e:
    ArkitektRekuest = NotAvailable
try:
    from arkitekt.apps.reaktion_rekuest import ArkitektRekuest
except ImportError as e:
    ArkitektRekuest = ArkitektRekuest  # Overwriting the non scheduling one
try:
    from arkitekt.apps.mikro import ArkitektMikro
except ImportError as e:
    ArkitektMikro = NotAvailable
try:
    from arkitekt.apps.mikro_next import ArkitektMikroNext
except ImportError as e:
    ArkitektMikroNext = NotAvailable
try:
    from arkitekt.apps.herre import ArkitektHerre
except ImportError as e:
    ArkitektHerre = NotAvailable
try:
    from arkitekt.apps.fluss import ArkitektFluss
except ImportError as e:
    ArkitektFluss = NotAvailable
try:
    from arkitekt.apps.unlok import ArkitektUnlok
except ImportError as e:
    ArkitektUnlok = NotAvailable
try:
    from arkitekt.apps.fakts import ArkitektFakts, Manifest
except ImportError as e:
    Manifest = Any
    ArkitektFakts = NotAvailable


class App(Composition):
    manifest: Any
    fakts: ArkitektFakts
    herre: ArkitektHerre
    rekuest: ArkitektRekuest
    mikro: ArkitektMikro
    mikro_next: ArkitektMikroNext
    fluss: ArkitektFluss
    unlok: ArkitektUnlok



from importlib import import_module
from arkitekt.cli.types import Manifest
from arkitekt.utils import create_arkitekt_folder
from pydantic import BaseModel, Field
import os
from typing import List, Optional
import yaml
import json
import datetime


def import_deployer(builder):
    module_path, function_name = builder.rsplit(".", 1)
    module = import_module(module_path)
    function = getattr(module, function_name)
    return function


def generate_definitions(module_path: str = "hu"):
    from rekuest.definition.registry import get_default_definition_registry

    module_path = "hu"
    import_module(module_path)

    reg = get_default_definition_registry()
    return list(reg.definitions.keys())


def load_manifest() -> Optional[Manifest]:
    path = create_arkitekt_folder()
    config_file = os.path.join(path, "manifest.yaml")
    if os.path.exists(config_file):
        with open(config_file, "r") as file:
            manifest = yaml.safe_load(file)
            if manifest:
                return Manifest(**manifest)
            else:
                return None
    return None


def write_manifest(manifest: Manifest):
    path = create_arkitekt_folder()
    config_file = os.path.join(path, "manifest.yaml")

    with open(config_file, "w") as file:
        yaml.safe_dump(
            json.loads(manifest.json(exclude_none=True, exclude_unset=True)),
            file,
            sort_keys=True,
        )



from pydantic import BaseModel, Field
from typing import List, Optional


class Manifest(BaseModel):
    version: str
    identifier: str
    scopes: List[str]
    logo: Optional[str]
    """ Scopes that this app should request from the user """

    class Config:
        extra = "forbid"


class User(BaseModel):
    id: str = Field(alias="sub")
    username: str = Field(alias="preferred_username")
    email: str = Field(alias="email")
    """ The user's preferred username """
